package com.qmth.themis.backend.api;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.themis.backend.config.DictionaryConfig;
import com.qmth.themis.backend.util.ServletUtil;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TBAttachment;
import com.qmth.themis.business.entity.TBPrivilege;
import com.qmth.themis.business.entity.TBUser;
import com.qmth.themis.business.enums.RoleEnum;
import com.qmth.themis.business.service.TBAttachmentService;
import com.qmth.themis.business.service.TBPrivilegeService;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 系统信息 前端控制器
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/27
 */
@Api(tags = "系统信息Controller")
@RestController
@RequestMapping("/${prefix.url.admin}/sys")
public class SysController {

    @Resource
    TBPrivilegeService tbPrivilegeService;

    @Resource
    TBAttachmentService tbAttachmentService;

    @Resource
    DictionaryConfig dictionaryConfig;

    @ApiOperation(value = "菜单查询接口")
    @RequestMapping(value = "/getMenu", method = RequestMethod.GET)
    @ApiResponses({@ApiResponse(code = 200, message = "菜单信息", response = TBPrivilege.class)})
    public Result getMenu(HttpServletRequest request) {
        TBUser tbUser = (TBUser) ServletUtil.getRequestAccount(request);
        if (Objects.isNull(tbUser)) {
            throw new BusinessException(ExceptionResultEnum.LOGIN_NO);
        }
        List<TBPrivilege> tbPrivilegeList = null;
        if (Objects.nonNull(tbUser.getRemark()) && Objects.equals(tbUser.getRemark(), RoleEnum.SUPER_ADMIN.getCode())) {
            QueryWrapper<TBPrivilege> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(TBPrivilege::getType, SystemConstant.MENU).orderByAsc(TBPrivilege::getParentId, TBPrivilege::getSequence);
            tbPrivilegeList = tbPrivilegeService.list(wrapper);
        } else {
            tbPrivilegeList = tbPrivilegeService.getMenu(tbUser.getId());
        }
        Map map = new HashMap();
        map.put(SystemConstant.RECORDS, tbPrivilegeList);
        return ResultUtil.ok(map);
    }

    @ApiOperation(value = "获取环境接口")
    @RequestMapping(value = "/env", method = RequestMethod.GET)
    @ApiResponses({@ApiResponse(code = 200, message = "环境信息", response = Result.class)})
    public Result env(@ApiParam(value = "机构id", required = true) @RequestParam Long orgId) {
        if (Objects.isNull(orgId) || Objects.equals(orgId, "")) {
            throw new BusinessException(ExceptionResultEnum.SCHOOL_ID_IS_NULL);
        }
//        SchoolDto schoolDto = (SchoolDto) redisTemplate.opsForValue().get(SystemConstant.SCHOOL_CACHE + orgId);
//        Map map = new HashMap();
//        map.put(SystemConstant.ENV_FILEHOST, schoolDto.getFileHost());
//        return ResultUtil.ok(map);
        return ResultUtil.ok(SystemConstant.SUCCESS);
    }

    @ApiOperation(value = "上传文件接口")
    @RequestMapping(value = "/file/upload", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "{\"id\":0}", response = Result.class)})
    public Result fileUpload(@ApiParam(value = "上传文件", required = true) @RequestParam MultipartFile file, HttpServletRequest request) throws IOException {
        Map<String, Object> mapParameter = new HashMap<>();
        mapParameter.put(SystemConstant.END_POINT, dictionaryConfig.aliYunOssDomain().getEndpoint());
        mapParameter.put(SystemConstant.ACCESS_KEY_ID, dictionaryConfig.aliYunOssDomain().getAccessKeyId());
        mapParameter.put(SystemConstant.ACCESS_KEY_SECRET, dictionaryConfig.aliYunOssDomain().getAccessKeySecret());
        mapParameter.put(SystemConstant.BUCKET, dictionaryConfig.aliYunOssDomain().getBucket());
        mapParameter.put(SystemConstant.OSS, dictionaryConfig.sysDomain().isOss());
        mapParameter.put(SystemConstant.ATTACHMENT_TYPE, dictionaryConfig.sysDomain().getAttachmentType());
        mapParameter.put(SystemConstant.LOCALHOST_PATH, dictionaryConfig.sysDomain().getLocalhostPath());
        mapParameter.put(SystemConstant.BASEPHOTO_PATH, dictionaryConfig.sysDomain().getBasePhotoPath());
        TBUser tbUser = (TBUser) ServletUtil.getRequestAccount(request);
        TBAttachment tbAttachment = tbAttachmentService.saveAttachment(file, request.getHeader(SystemConstant.MD5), request.getHeader(SystemConstant.PATH), mapParameter, tbUser.getOrgId(), tbUser.getId());
        if (Objects.isNull(tbAttachment)) {
            throw new BusinessException("上传文件失败");
        }
        Map map = new HashMap();
        map.put(SystemConstant.ID, tbAttachment.getId());
        map.put("name", tbAttachment.getName());
        map.put(SystemConstant.TYPE, tbAttachment.getType());
        return ResultUtil.ok(map);
    }
}
