package com.qmth.themis.backend.util;

import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Objects;

/**
 * @Description: http工具
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/4/10
 */
public class ServletUtil {

    /**
     * 输出错误
     *
     * @param response
     * @param code
     * @param message
     * @throws IOException
     */
    public static void responseError(HttpServletResponse response, String code, String message) throws IOException {
        Result result = ResultUtil.error(code, message);
        String json = JacksonUtil.parseJson(result);
        response.getWriter().print(json);
    }

    /**
     * 获取请求的accessToken
     *
     * @param httpRequest
     * @return
     */
    public static String getRequestToken(HttpServletRequest httpRequest) {
        // 从header中获取token
        String token = httpRequest.getHeader(SystemConstant.ACCESS_TOKEN);
        // 如果header中不存在token，则从参数中获取token
        if (Objects.isNull(token)) {
            token = httpRequest.getParameter(SystemConstant.ACCESS_TOKEN);
        }
        return token;
    }

    /**
     * 获取请求的platform
     *
     * @param httpRequest
     * @return
     */
    public static String getRequestPlatform(HttpServletRequest httpRequest) {
        // 从header中获取platform
        String platform = httpRequest.getHeader(Constants.HEADER_PLATFORM);
        // 如果header中不存在platform，则从参数中获取platform
        if (Objects.isNull(platform)) {
            platform = httpRequest.getParameter(Constants.HEADER_PLATFORM);
        }
        return platform;
    }

    /**
     * 获取请求的deviceId
     *
     * @param httpRequest
     * @return
     */
    public static String getRequestDeviceId(HttpServletRequest httpRequest) {
        // 从header中获取deviceId
        String deviceId = httpRequest.getHeader(Constants.HEADER_DEVICE_ID);
        // 如果header中不存在deviceId，则从参数中获取deviceId
        if (Objects.isNull(deviceId)) {
            deviceId = httpRequest.getParameter(Constants.HEADER_DEVICE_ID);
        }
        return deviceId;
    }

    /**
     * 获取请求的time
     *
     * @param httpRequest
     * @return
     */
    public static String getRequestTime(HttpServletRequest httpRequest) {
        // 从header中获取time
        String time = httpRequest.getHeader(Constants.HEADER_TIME);
        // 如果header中不存在time，则从参数中获取time
        if (Objects.isNull(time)) {
            time = httpRequest.getParameter(Constants.HEADER_TIME);
        }
        return time;
    }

    /**
     * 获取请求的Authorization
     *
     * @param httpRequest
     * @return
     */
    public static String getRequestAuthorization(HttpServletRequest httpRequest) {
        // 从header中获取authorization
        String authorization = httpRequest.getHeader(Constants.HEADER_AUTHORIZATION);
        // 如果header中不存在authorization，则从参数中获取authorization
        if (Objects.isNull(authorization)) {
            authorization = httpRequest.getParameter(Constants.HEADER_AUTHORIZATION);
        }
        return authorization;
    }

    /**
     * 获取请求的Session
     *
     * @param httpRequest
     * @return
     */
    public static Object getRequestSession(HttpServletRequest httpRequest) {
        return httpRequest.getAttribute(SystemConstant.SESSION);
    }

    /**
     * 获取请求的account
     *
     * @param httpRequest
     * @return
     */
    public static Object getRequestAccount(HttpServletRequest httpRequest) {
        return httpRequest.getAttribute(SystemConstant.ACCOUNT);
    }

    /**
     * 获取请求的org
     *
     * @param httpRequest
     * @return
     */
    public static Object getRequestOrg(HttpServletRequest httpRequest) {
        return httpRequest.getAttribute(SystemConstant.ORG);
    }
}
