package com.qmth.themis.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.AuthDto;
import com.qmth.themis.business.entity.*;
import com.qmth.themis.business.enums.RoleEnum;
import com.qmth.themis.business.service.*;
import com.qmth.themis.business.util.EhcacheUtil;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Description: ehcache 服务实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/27
 */
@Service
public class EhcacheServiceImpl implements EhcacheService {
    private final static Logger log = LoggerFactory.getLogger(EhcacheServiceImpl.class);

    @Resource
    TBUserRoleService tbUserRoleService;

    @Resource
    TBPrivilegeService tbPrivilegeService;

    @Resource
    TBRolePrivilegeService tbRolePrivilegeService;

    @Resource
    TBUserService tbUserService;

    @Resource
    TBOrgService tbOrgService;

    /**
     * 添加用户缓存
     *
     * @param o
     * @return
     */
    @Override
    public AuthDto addAccountCache(Object o) {
        AuthDto authDto = null;
        try {
            TBUser user = null;
            if (o instanceof TBUser) {
                user = (TBUser) o;
            } else {
                user = tbUserService.getById(Long.parseLong(String.valueOf(o)));
            }
            if (Objects.isNull(user)) {
                throw new BusinessException(ExceptionResultEnum.USER_NO);
            }
            //查询用户角色和权限
            QueryWrapper<TBUserRole> uWrapper = new QueryWrapper<>();
            uWrapper.lambda().eq(TBUserRole::getUserId, user.getId());
            List<TBUserRole> tbUserRoleList = tbUserRoleService.list(uWrapper);
            if (Objects.nonNull(tbUserRoleList)) {
                //是否是系统管理员
                long count = tbUserRoleList.stream().filter(s -> Objects.equals(s.getRoleCode(), RoleEnum.SUPER_ADMIN.name())).count();
                if (count > 0) {
                    //系统管理员获取所有权限
                    QueryWrapper<TBPrivilege> pWrapper = new QueryWrapper<>();
                    pWrapper.lambda().eq(TBPrivilege::getType, SystemConstant.LINK);
                    List<TBPrivilege> tbPrivilegeList = tbPrivilegeService.list(pWrapper);
                    authDto = new AuthDto(tbUserRoleList.stream().map(s -> s.getRoleCode()).collect(Collectors.toSet()), tbPrivilegeList.stream().map(s -> s.getUrl()).collect(Collectors.toSet()));
                    EhcacheUtil.put(SystemConstant.AUTH_CACHE, user.getId(), authDto);
                } else {
                    TBOrg tbOrg = tbOrgService.getById(user.getOrgId());
                    //根据角色名查权限
                    Set<String> roleCodes = tbUserRoleList.stream().map(s -> s.getRoleCode()).collect(Collectors.toSet());
                    QueryWrapper<TBRolePrivilege> rpWrapper = new QueryWrapper<>();
                    rpWrapper.lambda().in(TBRolePrivilege::getRoleCode, roleCodes);
                    List<TBRolePrivilege> tbRolePrivilegeList = tbRolePrivilegeService.list(rpWrapper);
                    Set<Long> privilegeIds = tbRolePrivilegeList.stream().map(s -> s.getPrivilegeId()).collect(Collectors.toSet());
                    QueryWrapper<TBPrivilege> pWrapper = new QueryWrapper<>();
                    pWrapper.lambda().in(TBPrivilege::getId, privilegeIds).eq(TBPrivilege::getType, SystemConstant.LINK);
                    List<TBPrivilege> tbPrivilegeList = tbPrivilegeService.list(pWrapper);
                    authDto = new AuthDto(roleCodes, tbPrivilegeList.stream().map(s -> s.getUrl()).collect(Collectors.toSet()), tbOrg);
                    EhcacheUtil.put(SystemConstant.AUTH_CACHE, user.getId(), authDto);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException("添加用户缓存失败");
        }
        return authDto;
    }

    /**
     * 删除用户缓存
     *
     * @param userId
     */
    @Override
    public void removeAccountCache(Long userId) {
        EhcacheUtil.remove(SystemConstant.AUTH_CACHE, userId);
    }
}
