package com.qmth.themis.mq.listener;

import com.qmth.themis.business.constant.SystemConstant;
import org.apache.rocketmq.client.consumer.DefaultMQPushConsumer;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyContext;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyStatus;
import org.apache.rocketmq.client.consumer.listener.MessageListenerConcurrently;
import org.apache.rocketmq.common.message.Message;
import org.apache.rocketmq.common.message.MessageExt;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.apache.rocketmq.spring.core.RocketMQPushConsumerLifecycleListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description: 事务消息监听
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/28
 */
@Service
public class RocketConsumerTranListener {
    private final static Logger log = LoggerFactory.getLogger(RocketConsumerTranListener.class);

    @Service
    @RocketMQMessageListener(consumerGroup = "themis-tran-group", topic = "themis-tran-topic")
    public class consumerTranListener implements RocketMQListener<Message>, RocketMQPushConsumerLifecycleListener {

        @Override
        public void onMessage(Message message) {
            //实现RocketMQPushConsumerLifecycleListener监听器之后，此方法不调用
            log.info("实现RocketMQPushConsumerLifecycleListener监听器之后，此方法不调用");
        }

        @Override
        public void prepareStart(DefaultMQPushConsumer defaultMQPushConsumer) {
            defaultMQPushConsumer.setConsumeMessageBatchMaxSize(SystemConstant.CONSUME_MESSAGE_BATCH_MAX_SIZE);//每次拉取10条
            defaultMQPushConsumer.setMaxReconsumeTimes(SystemConstant.MAXRECONSUMETIMES);//最大重试次数
            defaultMQPushConsumer.registerMessageListener(new MessageListenerConcurrently() {
                @Override
                public ConsumeConcurrentlyStatus consumeMessage(List<MessageExt> msgs, ConsumeConcurrentlyContext context) {
                    for (MessageExt messageExt : msgs) {
                        log.info("重试次数:{}", messageExt.getReconsumeTimes());
                        log.info("接受到的消息:{}", new String(messageExt.getBody()));
                        return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;
                    }
                    return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;//成功
//                    return ConsumeConcurrentlyStatus.RECONSUME_LATER;//重试
                }
            });
        }
    }
}
