package com.qmth.themis.task.quartz;

import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.threadPool.MyThreadPool;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.mq.dto.MqDto;
import com.qmth.themis.mq.service.ProducerServer;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.quartz.QuartzJobBean;

import javax.annotation.Resource;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: mq job
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/1
 */
public class MqJob extends QuartzJobBean {
    private final static Logger log = LoggerFactory.getLogger(MqJob.class);

    @Resource
    RedisUtil redisUtil;

    @Resource
    ProducerServer producerServer;

    @Resource
    MyThreadPool myThreadPool;

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        myThreadPool.arbitratePoolTaskExecutor.execute(() -> {
            log.info("session_topic_job进来了");
            this.assembleJob(SystemConstant.SESSION_TOPIC_BUFFER_LIST);
        });
        myThreadPool.arbitratePoolTaskExecutor.execute(() -> {
            log.info("userLog_topic_job进来了");
            this.assembleJob(SystemConstant.USERLOG_TOPIC_BUFFER_LIST);
        });
    }

    public void assembleJob(String redisKey) {
        Long size = redisUtil.getHashSize(redisKey);
        if (Objects.nonNull(size) && size.longValue() > 0) {
            log.info("redisKey:{}缓冲区的消息数为:{}", redisKey, size);
            Map map = redisUtil.getHashEntries(redisKey);
            map.forEach((k, v) -> {
                MqDto mqDto = (MqDto) v;
                producerServer.sendOneWay(mqDto);
            });
        }
    }
}
